/*--------------------------------------------------------------------------
 *  Copyright 2008 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// HTMLWriter.java
// Since: Apr 22, 2008 2:35:39 PM
//
// $URL: https://www.xerial.org/svn/project/XerialJ/trunk/xerial-wiki/src/main/java/org/xerial/wiki/HTMLWriter.java $
// $Author: leo $
//--------------------------------------
package org.xerial.wiki;

import java.io.IOException;
import java.io.Writer;

import org.xerial.wiki.html.HTMLAttribute;

/**
 * HTML generation class
 * 
 * @author leo
 * 
 */
public class HTMLWriter
{
    private final Writer writer;
    private boolean allowUnescapedText = false;

    public HTMLWriter(Writer writer)
    {
        this.writer = writer;
    }

    public void startTag(String name) throws IOException
    {
        startTag(name, null);
    }

    public void startTag(String name, HTMLAttribute attribute) throws IOException
    {
        writer.append("<");
        writer.append(name);
        if (attribute != null && attribute.size() > 0)
        {
            // output attributes
            StringBuilder buf = new StringBuilder();
            attribute.toHTML(buf);
            writer.append(" ");
            writer.append(buf.toString());
        }
        writer.append(">");
    }

    public void selfCloseTag(String name) throws IOException
    {
        writer.append("<");
        writer.append(name);
        writer.append("/>");
    }

    public void endTag(String name) throws IOException
    {
        writer.append("</");
        writer.append(name);
        writer.append(">");
    }

    public void text(String text) throws IOException
    {
        if (allowUnescapedText)
            writer.append(text);
        else
            writer.append(escapeText(text));
    }

    public void rawText(String text) throws IOException
    {
        writer.append(text);
    }

    public static String escapeText(String s)
    {
        s = s.replaceAll("&", "&amp;");
        s = s.replaceAll("<", "&lt;");
        s = s.replaceAll(">", "&gt;");
        s = s.replaceAll("\"", "&quot;");
        s = s.replaceAll("'", "&apos;");
        return s;
    }

    public void allowUnescapedText(boolean allow)
    {
        allowUnescapedText = allow;
    }

}
