/*--------------------------------------------------------------------------
 *  Copyright 2008 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// Wiki.java
// Since: Apr 22, 2008 10:12:53 AM
//
// $URL: https://www.xerial.org/svn/project/XerialJ/trunk/xerial-wiki/src/main/java/org/xerial/wiki/Wiki.java $
// $Author: leo $
//--------------------------------------
package org.xerial.wiki;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;

import org.antlr.runtime.ANTLRReaderStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.RecognitionException;
import org.antlr.runtime.tree.CommonTree;
import org.xerial.core.XerialException;
import org.xerial.util.StringUtil;
import org.xerial.util.bean.impl.BeanUtilImpl;
import org.xerial.util.log.Logger;
import org.xerial.wiki.impl.WikiLexer;
import org.xerial.wiki.impl.WikiParser;

/**
 * Wiki data object
 * 
 * @author leo
 * 
 */
public class Wiki implements WikiElement
{
    private static Logger _logger = Logger.getLogger(Wiki.class);
    private ArrayList<WikiElement> wikiElementList = new ArrayList<WikiElement>();

    public void Wiki()
    {}

    public WikiElementType getType()
    {
        return WikiElementType.Root;
    }

    public void addWikiLine(WikiLine wikiLine)
    {
        wikiElementList.add(wikiLine);
    }

    public void addBlankLine(String line)
    {
        wikiElementList.add(new BlankLine());
    }

    public void addWikiBlock(WikiBlock block)
    {
        wikiElementList.add(block);
    }

    public static Wiki load(Reader wikiTextReader) throws IOException, WikiException
    {
        WikiLexer lexer = new WikiLexer(new ANTLRReaderStream(wikiTextReader));
        CommonTokenStream tokens = new CommonTokenStream(lexer);
        _logger.trace(StringUtil.join(tokens.getTokens(), "\n"));
        WikiParser parser = new WikiParser(tokens);
        try
        {
            WikiParser.wiki_return ret = parser.wiki();
            CommonTree t = (CommonTree) ret.getTree();
            _logger.debug(WikiUtil.parseTree(t));
            Wiki wiki = BeanUtilImpl.createBeanFromParseTree(Wiki.class, t, WikiParser.tokenNames);
            return wiki;
        }
        catch (RecognitionException e)
        {
            throw new WikiException(e);
        }
        catch (XerialException e)
        {
            throw new WikiException(WikiErrorCode.WikiBindingError, e.getMessage());
        }

    }

    public static Wiki load(InputStream wikiTextStream) throws WikiException, IOException
    {
        return load(new InputStreamReader(wikiTextStream));
    }

    /**
     * Generates an HTML output of this wiki instance
     * 
     * @return
     * @throws IOException
     * @throws WikiException
     */
    public String toHTML() throws IOException, WikiException
    {

        WikiToHTMLWriter htmlWriter = new WikiToHTMLWriter();
        WikiWalker walker = new WikiWalker(htmlWriter);
        walker.walk(this);

        StringWriter writer = new StringWriter();
        htmlWriter.render(writer);
        return writer.toString();
    }

    /**
     * Walk the wiki document and generate the output to the given
     * {@link WikiWriter}
     * 
     * @param walker
     * @throws WikiException
     */
    public void walk(WikiWriter writer) throws WikiException
    {
        WikiWalker walker = new WikiWalker(writer);
        walker.walk(this);
    }

    public List<WikiElement> getWikiElementList()
    {
        return wikiElementList;
    }

}
