/*--------------------------------------------------------------------------
 *  Copyright 2007 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// WikiException.java
// Since: Sep 15, 2007 6:20:35 PM
//
// $URL$
// $Author$
//--------------------------------------
package org.xerial.wiki;

import org.antlr.runtime.RecognitionException;
import org.xerial.core.XerialException;

/**
 * WikiException
 * 
 * @author leo
 * 
 */
public class WikiException extends XerialException
{
    private static final long serialVersionUID = 3254941962788457552L;

    private int line = -1;
    private final WikiErrorCode errorCode;

    public WikiException(WikiErrorCode errorCode, String message)
    {
        super(message);
        this.errorCode = errorCode;
    }

    public WikiException(RecognitionException e)
    {
        this(e.line, e + " token=" + e.token + " line: " + e.line + "(" + e.charPositionInLine + ")");
    }

    public WikiException(int line, String message)
    {
        super(message);
        this.errorCode = WikiErrorCode.SyntaxError;
        this.line = line;
    }

    public WikiErrorCode getErrorCode()
    {
        return errorCode;
    }

    public int getLineExceptionOccured()
    {
        return line;
    }

}
