/*--------------------------------------------------------------------------
 *  Copyright 2008 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// WikiFunction.java
// Since: Apr 22, 2008 10:21:50 AM
//
// $URL: https://www.xerial.org/svn/project/XerialJ/trunk/xerial-wiki/src/main/java/org/xerial/wiki/WikiFunction.java $
// $Author: leo $
//--------------------------------------
package org.xerial.wiki;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * Wiki function data type
 * 
 * @author leo
 * 
 */
public class WikiFunction implements WikiElement
{
    /**
     * Function name
     */
    private String name = null;
    private ArrayList<WikiElement> argumentList = new ArrayList<WikiElement>();
    private TreeMap<String, String> assignment = new TreeMap<String, String>();

    public WikiFunction()
    {}

    public String getName()
    {
        return name;
    }

    public void setName(String name)
    {
        this.name = name;
    }

    /**
     * Add string type function argument
     * 
     * @param value
     */
    public void addArgument(String value)
    {
        argumentList.add(new WikiText(value));
    }

    public void addFunction(WikiFunction function)
    {
        argumentList.add(function);
    }

    public void putAssignment(String key, String value)
    {
        assignment.put(key, value);
    }

    public WikiElementType getType()
    {
        return WikiElementType.Function;
    }

    public List<WikiElement> getArgumentList()
    {
        return argumentList;
    }

    public Map<String, String> getAssignment()
    {
        return assignment;
    }

}
