/*--------------------------------------------------------------------------
 *  Copyright 2007 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// WikiHash.java
// Since: Sep 16, 2007 11:50:59 AM
//
// $URL$
// $Author$
//--------------------------------------
package org.xerial.wiki;

import java.util.HashMap;
import java.util.List;
import java.util.Set;

import org.xerial.json.JSONObject;
import org.xerial.json.JSONValue;

/**
 * hash representation of WikiValue
 * @author leo
 *
 */
public class WikiHash implements WikiValue
{
    HashMap<String, WikiValue> hash = new HashMap<String, WikiValue>();
    
    public WikiHash()
    {}
    
    /**
     * Puts a parameter value to this hash 
     * @param parameterName
     * @param value
     */
    public void put(String parameterName, WikiValue value)
    {
        hash.put(parameterName, value);
    }
    public boolean containsKey(String parameterName)
    {
        return hash.containsKey(parameterName);
    }

    public WikiValue getValue(String parameterName)
    {
        return hash.get(parameterName);
    }
    
    public WikiValue getValue(String parameterName, WikiValue defaultValue)
    {
        return containsKey(parameterName) ? hash.get(parameterName) : defaultValue;
    }
    
    public Set<String> keySet()
    {
        return hash.keySet();
    }
    
    
    public WikiArray isArray()
    {
        return null;
    }

    public WikiHash isHash()
    {
        return this;
    }

    public String isString()
    {
        return null;
    }

    public JSONObject toJSON(String parameterName)
    {
        JSONObject json = new JSONObject();
        json.put(parameterName, toJSONValue());
        return json;
    }

    public JSONValue toJSONValue()
    {
        JSONObject json = new JSONObject();
        for(String key : keySet())
        {
            json.put(key, hash.get(key).toJSONValue());
        }
        return json;
    }
    
    public String toString()
    {
        return hash.toString();
    }

}
