/*--------------------------------------------------------------------------
 *  Copyright 2007 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// WikiPluginLoader.java
// Since: Sep 16, 2007 6:41:02 PM
//
// $URL$
// $Author$
//--------------------------------------
package org.xerial.wiki;

import java.util.HashMap;
import java.util.List;

import org.xerial.util.FileResource;
import org.xerial.util.ResourceFilter;
import org.xerial.util.io.VirtualFile;
import org.xerial.util.log.Logger;

/**
 * Plugin loader
 * 
 * @author leo
 * 
 */
public class WikiPluginLoader
{
    private static Logger _logger = Logger.getLogger(WikiPluginLoader.class);

    private HashMap<String, Class<WikiPlugin>> pluginTable = new HashMap<String, Class<WikiPlugin>>();

    public WikiPluginLoader()
    {
        scanPlugins();
    }

    public WikiPlugin getPlugin(String name)
    {
        if (name == null)
            return null;
        Class<WikiPlugin> pluginClass = pluginTable.get(name.toLowerCase());
        try
        {
            if (pluginClass != null)
                return pluginClass.newInstance();
            else
                return null;
        }
        catch (InstantiationException e)
        {
            _logger.error(e);
        }
        catch (IllegalAccessException e)
        {
            _logger.error(e);
        }
        return null;
    }

    @SuppressWarnings("unchecked")
    private void scanPlugins()
    {
        String pluginPackage = "org.xerial.wiki.plugin";
        List<VirtualFile> pluginClassList = FileResource.listResources(pluginPackage, new ResourceFilter() {
            public boolean accept(String resourcePath)
            {
                return resourcePath.endsWith(".class");
            }
        });
        _logger.trace("found " + pluginClassList.size() + " classes");

        for (VirtualFile vf : pluginClassList)
        {
            String className = pluginPackage + "." + vf.getLogicalPath().replaceAll("/", ".").replace(".class", "");
            try
            {
                Class foundClass = Class.forName(className);
                if (WikiPlugin.class.isAssignableFrom(foundClass))
                {
                    String pluginClassName = foundClass.getName();
                    _logger.trace("found a plugin: " + pluginClassName);

                    int pos = pluginClassName.lastIndexOf(".");
                    String pluginClassNameBase = (pos > 0) ? pluginClassName.substring(pos + 1) : pluginClassName;
                    String pluginFunctionName = pluginClassNameBase.toLowerCase().replace("plugin", "");

                    Class<WikiPlugin> pluginClass = (Class<WikiPlugin>) foundClass;
                    _logger.trace("function: " + pluginFunctionName + ", class: " + pluginClass.getName());
                    pluginTable.put(pluginFunctionName, pluginClass);
                }
            }
            catch (ClassNotFoundException e)
            {
                // skip the not found class
            }

        }

    }

}
