/*--------------------------------------------------------------------------
 *  Copyright 2008 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// WikiVisitor.java
// Since: Apr 22, 2008 1:32:06 PM
//
// $URL: https://www.xerial.org/svn/project/XerialJ/trunk/xerial-wiki/src/main/java/org/xerial/wiki/WikiWriter.java $
// $Author: leo $
//--------------------------------------
package org.xerial.wiki;

import java.io.Writer;

/**
 * The writer interface for converting Wiki text into a some specific document
 * format.
 * 
 * @author leo
 * 
 */
public interface WikiWriter
{
    /**
     * Render the whole document into the writer
     * 
     * @param writer
     * @throws WikiException
     */
    void render(Writer writer) throws WikiException;

    /**
     * Set the title of the page.
     * 
     * @param title
     */
    void setTitle(String title) throws WikiException;

    /**
     * Beginning the section element.
     * 
     * @param level
     *            section level 1 to 3.
     */
    void beginSection(int level) throws WikiException;

    /**
     * Ending the section element.
     */
    void endSection() throws WikiException;

    /**
     * Beginning the item element.
     * 
     * @param level
     */
    void beginItem(int level) throws WikiException;

    /**
     * Ending the item element;
     */
    void endItem() throws WikiException;

    /**
     * Adding a text.
     * 
     * @param text
     */
    void text(String text) throws WikiException;

    /**
     * Beginning the bold text.
     */
    void beginBold() throws WikiException;

    /**
     * Ending the bold text.
     */
    void endBold() throws WikiException;

    /**
     * Beginning the Italic text.
     */
    void beginItaric() throws WikiException;

    /**
     * Ending the Italic text.
     */
    void endItaric() throws WikiException;

    /**
     * Beginning the verbatim (pre) element.
     * 
     */
    void beginVerbatim() throws WikiException;

    /**
     * Ending the verbatim (pre) element.
     * 
     */
    void endVerbatim() throws WikiException;

    void beginBody() throws WikiException;

    void endBody() throws WikiException;

    /**
     * Beginning the link element.
     * 
     * @param ref
     *            reference to a web page
     * @param target
     *            target browser window. This target value can be null.
     */
    void beginLink(String ref, String target) throws WikiException;

    /**
     * Ending the link element.
     */
    void endLink() throws WikiException;

    /**
     * Beginning the anchor (<a name="<name>">... )
     * 
     * @param name
     * @throws WikiException
     */
    void beginAnchor(String name) throws WikiException;

    /**
     * Ending the anchor
     */
    void endAnchor() throws WikiException;

    void newLine() throws WikiException;

    void beginParagraph() throws WikiException;

    void endParagraph() throws WikiException;

    void beginSectionTitle(int level) throws WikiException;

    void endSectionTitle(int level) throws WikiException;

    void beginImage(String href, String label) throws WikiException;

    void endImage() throws WikiException;

    public enum ListType {
        UNORDERED, ORDERED;
    }

    /**
     * Adjust the indentation of list items
     * 
     * @param listType
     * @param targetLevel
     * @throws WikiException
     */
    void adjustIndent(ListType listType, int targetLevel) throws WikiException;

    /**
     * Reset the indentation of list items to the target level
     * 
     * @throws WikiException
     */
    void resetIndent(int targetLevel) throws WikiException;

    /**
     * Beginning the unordered list (ul)
     * 
     * @throws WikiException
     */
    void beginUnorderedList() throws WikiException;

    /**
     * Ending the unordered list (ul)
     * 
     * @throws WikiException
     */
    void endUnorderedList() throws WikiException;

    /**
     * Beginning the ordered list (ol)
     * 
     * @throws WikiException
     */
    void beginOrderedList() throws WikiException;

    /**
     * Ending the ordered list (ol)
     * 
     * @throws WikiException
     */
    void endOrderedList() throws WikiException;

    /**
     * Addina a raw text without escaping some characters
     * 
     * @param text
     * @throws WikiException
     */
    void rawText(String text) throws WikiException;

}
