/*--------------------------------------------------------------------------
 *  Copyright 2008 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// WikiWriterBase.java
// Since: Apr 23, 2008 10:29:44 AM
//
// $URL$
// $Author$
//--------------------------------------
package org.xerial.wiki;

import java.util.LinkedList;

/**
 * Abstract base class of the {@link WikiWriter}. This class contains common
 * processing patterns of list item, section handling, etc.
 * 
 * @author leo
 * 
 */
public abstract class WikiWriterBase implements WikiWriter
{

    protected final LinkedList<ListType> listTypeStack = new LinkedList<ListType>();
    private boolean isInVerbatimBlock = false;

    protected WikiWriterBase()
    {

    }

    public void adjustIndent(ListType listType, int targetLevel) throws WikiException
    {
        int currentLevel = listTypeStack.size();
        while (currentLevel < targetLevel)
        {
            switch (listType)
            {
            case UNORDERED:
                beginUnorderedList();
                break;
            case ORDERED:
                beginOrderedList();
                break;
            }
            listTypeStack.add(listType);
            currentLevel++;
        }
        resetIndent(targetLevel);
    }

    public void resetIndent(int targetLevel) throws WikiException
    {
        int currentLevel = listTypeStack.size();
        while (currentLevel > targetLevel)
        {
            ListType parentListType = listTypeStack.removeLast();
            switch (parentListType)
            {
            case UNORDERED:
                endUnorderedList();
                break;
            case ORDERED:
                endOrderedList();
                break;
            }
            currentLevel--;
        }
    }

}
