/*--------------------------------------------------------------------------
 *  Copyright 2007 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// HTMLElement.java
// Since: Sep 16, 2007 1:16:04 PM
//
// $URL$
// $Author$
//--------------------------------------
package org.xerial.wiki.html;

import java.util.ArrayList;

public class HTMLElement
{
    private String tagName;
    private HTMLAttribute attribute = null;
    private ArrayList<Object> contentList = new ArrayList<Object>();
    
    public HTMLElement(String tagName)
    {
        this(tagName, null);
    }
    
    public HTMLElement(String tagName, HTMLAttribute attribute)
    {
        this.tagName = tagName;
        this.attribute = attribute;
    }
    public void addAttribute(String attributeName, Object value)
    {
        if(attribute == null)
            attribute = new HTMLAttribute();
        attribute.add(attributeName, value);
    }
    
    public void addContent(Object content)
    {
        contentList.add(content);
    }
    
    public void toHTML(StringBuilder out)
    {
        out.append("<");
        out.append(tagName);
        if(attribute != null)
        {
            out.append(" ");
            attribute.toHTML(out);
        }
        
        if(contentList.size() == 0)
        {
            // self close tag
            out.append("/>");
        }
        else
        {
            out.append(">");
            for(Object content : contentList)
            {
                if(content == null)
                    continue;
                
                out.append(content.toString());
            }
            
            // close the tag
            out.append("</");
            out.append(tagName);
            out.append(">");
        }
            
    }
    
    public String toString()
    {
        StringBuilder builder = new StringBuilder();
        toHTML(builder);
        return builder.toString();
    }
    
}
