/*--------------------------------------------------------------------------
 *  Copyright 2007 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------------------
// xerial-wiki project
// 
// Wiki.g
// Since  2007/09/08 23:17:42
//  
//--------------------------------------------------
grammar Wiki;
options 
{
	language=Java;
	output=AST;	
}

tokens {
Wiki;
WikiLine;
WikiBlock;
Text;
Path;
Function;
Name;
Assignment;
Argument;
Key;
Value;
String;
Array;
Hash;
XMLElement;
XMLAttribute;
}


@lexer::header
{
/*--------------------------------------------------------------------------
 *  Copyright 2007 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------------------
// xerial-wiki project
// 
// WikiLexer.java
// Since  2007/09/08 23:17:42
// 
//--------------------------------------------------

package org.xerial.wiki.impl;

import java.util.Iterator;
import java.util.HashMap;
import java.util.Properties;
import org.xerial.util.FileResource;
import org.xerial.util.log.Logger;
import java.io.IOException;
import java.net.URL;
}

@lexer::members
{
  	private static HashMap<Integer, String> tokenTable = new HashMap<Integer, String>();
  	private static Logger _logger = Logger.getLogger(WikiLexer.class);
  	static
  	{
  		Properties p = new Properties();
  		try
        {
   		    URL wikiTokenFileURL = FileResource.find(WikiLexer.class, "Wiki.tokens");
   		    p.load(wikiTokenFileURL.openStream());

             for(Iterator it = p.keySet().iterator(); it.hasNext(); )
             {
				String tokenName = (String) it.next();
   	            int tokenType = Integer.parseInt(p.get(tokenName).toString());
                tokenTable.put(tokenType, tokenName);
             }
        }
        catch (IOException e)
        {
           _logger.error(e);
        }
  	}
  	public static String getTokenName(int tokenType)
   	{
   		return tokenTable.containsKey(tokenType) ? tokenTable.get(tokenType) : "UNKNOWN";
   	}
}

@header
{
/*--------------------------------------------------------------------------
 *  Copyright 2007 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------------------
// xerial-wiki project
// 
// WikiParser.java
// Since  2007/09/08 23:17:42
// 
//--------------------------------------------------
package org.xerial.wiki.impl;
}


@rulecatch { 
catch(RecognitionException e){
	throw e;
}
} 



ML_COMMENT
	:   '/*' (options {greedy=false;} : .)* '*/' { channel=HIDDEN; }
 	;

LINE_COMMENT
    : {getCharPositionInLine()==0}? => '//' ~('\n'|'\r')* NEWLINE { channel=HIDDEN; }
    ;

NEWLINE
	: '\r'? '\n'
	;

 
fragment UpperLetter : 'A' .. 'Z';
fragment LowerLetter: 'a' .. 'z';
fragment Letter : UpperLetter | LowerLetter;
fragment Digit: '0' .. '9';
fragment HexDigit: ('0' .. '9' | 'A' .. 'F' | 'a' .. 'f');
fragment UnicodeChar: ~('"'| '\\');
fragment StringChar :  UnicodeChar | EscapeSequence;

fragment EscapeSequence
	: '\\' ('\"' | '\\' | '/' | 'b' | 'f' | 'n' | 'r' | 't' | 'u' HexDigit HexDigit HexDigit HexDigit)
	;


fragment Int: '-'? ('0' | '1'..'9' Digit*);
fragment Frac: Dot Digit+;
fragment Exp: ('e' | 'E') ('+' | '-')? Digit+;

fragment WhiteSpace: (' ' | '\t' | '\u000C');

LineBreak: '<BR>' (WhiteSpaces* NEWLINE)?;

//BlankLine: {getCharPositionInLine()==0}? => WhiteSpace* (NEWLINE | EOF);

WhiteSpaces: WhiteSpace+;

H1: {getCharPositionInLine()==0}? => '=' { setText("H1"); };
H2: {getCharPositionInLine()==0}? => '==' { setText("H2"); };
H3: {getCharPositionInLine()==0}? => '===' { setText("H3"); };

I1: {getCharPositionInLine()==0}? => '*' { setText("I1"); };
I2: {getCharPositionInLine()==0}? => '**' { setText("I2"); };
I3: {getCharPositionInLine()==0}? => '***' { setText("I3"); };

O1: {getCharPositionInLine()==0}? => '+' { setText("I1"); };
O2: {getCharPositionInLine()==0}? => '++' { setText("I2"); };
O3: {getCharPositionInLine()==0}? => '+++' { setText("I3"); };




fragment NameChar: Letter | Digit | Dot | '-' | '_' ;

fragment Name: Letter NameChar*;

//StartTag: Lt n=Name Gt { setText(n.getText()); }; 
//EndTag: Lt Slash n=Name Gt { setText(n.getText()); };




StringLiteral: '"' body=StringCharacters '"' { setText($body.text); };
fragment StringCharacters: StringChar*;

fragment
ProtocolPrefix
	: ('http' | 'ftp' | 'https' | 'sftp' | 'rsync' ) '://'
	;
	
Dot: '.';
LParen: '(';
RParen: ')';
LBracket: '[';
RBracket: ']';
LBrace: '{';
RBrace: '}';
Lt: '<';
Gt: '>';
Slash: '/';
At: '@';
Comma: ',';
Colon: ':';
Eq: '=';
Star: '*';
//And: '&';
//Question: '?';
//Exclamation: '!';
//Sharp: '#';

fragment
Symbol
	: LBracket | RBracket | LBrace | RBrace | LParen | RParen 
	| Lt | Gt | At | Comma | Eq  | '"' | Star  | Slash
//	| And | Question | Exclamation | Sharp
	;

fragment
WikiBlockSymbol
	: Star | Eq
	;


fragment
URLChar: NameChar | At | Dot 
	;

fragment
URLQueryChar: URLChar | Eq | '%'  
	;

CloseTagBegin: Lt Slash; 



fragment
WikiNameUnit: UpperLetter+ LowerLetter+ 
	;


WikiName
	: WikiNameUnit WikiNameUnit
	| WikiNameUnit+ (Slash WikiNameUnit+)+ WikiNameUnit WikiNameUnit
	;

InternetLink
	: (ProtocolPrefix) => ProtocolPrefix (~( WhiteSpace | '\n' | '\r' | ';' | LParen | RParen) | Digit)*
	| (~('!')) => Slash? URLChar+ (Colon Digit+)? (Slash URLChar*)* ('?' URLQueryChar*)?
	;	



fragment FunctionID: Name (Colon Name)?;

FunctionName: (~('!')) => At FunctionID LParen { setText($FunctionID.text); } 	
	; 

	

Word
	: {getCharPositionInLine() > 0 }? => (~(Symbol | WhiteSpace | '\n' | '\r'))+
	| {getCharPositionInLine() == 0}? => ~(WikiBlockSymbol | Symbol) (~(Symbol | WhiteSpace | '\n' | '\r'))*
	| ('!' WikiName) => '!' WikiName  { setText(getText().substring(1)); }
	| ('!' InternetLink) => '!' InternetLink  { setText(getText().substring(1)); }
	;
	



PreText
	: '<pre>' (options {greedy=false;} : .)* '</pre>'
	{ 
	  String tmp = getText(); 
	  setText(tmp.substring(5, tmp.length()-6));
	}
	| '{{' (options {greedy=false;} : .)* '}}'
	{
	  String tmp = getText(); 
	  setText(tmp.substring(2, tmp.length()-2));
	}
	;

/**
 * Parser grammar
 */ 




wiki
	: wikiExpr*	wikiFragment -> ^(Wiki wikiExpr* wikiFragment)
	;

	
wikiExpr
	: inlineWiki -> ^(WikiLine inlineWiki)
	| blockWiki -> ^(WikiBlock blockWiki)
//	| BlankLine
	;	

wikiFragment
	: wikiText* EOF -> ^(WikiLine wikiText*)
	;


blockWiki
	: H1 inlineWiki 
	-> ^(Function Name["BeginSectionTitle"] ^(Assignment Key["level"] Value["1"])) 
		inlineWiki 
	   ^(Function Name["EndSectionTitle"] ^(Assignment Key["level"] Value["1"])) 
	| H2 inlineWiki 
	-> ^(Function Name["BeginSectionTitle"] ^(Assignment Key["level"] Value["2"])) 
		inlineWiki 
		^(Function Name["EndSectionTitle"] ^(Assignment Key["level"] Value["2"]))
	| H3 inlineWiki 
	-> ^(Function Name["BeginSectionTitle"] ^(Assignment Key["level"] Value["3"])) 
		inlineWiki 
		^(Function Name["EndSectionTitle"] ^(Assignment Key["level"] Value["3"]))
	| I1 inlineWiki 
	-> ^(Function Name["BeginUnorderedListItem"] ^(Assignment Key["level"] Value["1"])) 
		inlineWiki 
		^(Function Name["EndUnorderedListItem"])
	| I2 inlineWiki 
	-> ^(Function Name["BeginUnorderedListItem"] ^(Assignment Key["level"] Value["2"])) 
		inlineWiki 
		^(Function Name["EndUnorderedListItem"])
	| I3 inlineWiki 
	-> ^(Function Name["BeginUnorderedListItem"] ^(Assignment Key["level"] Value["3"])) 
		inlineWiki 
		^(Function Name["EndUnorderedListItem"])
	| O1 inlineWiki 
	-> ^(Function Name["BeginOrderedListItem"] ^(Assignment Key["level"] Value["1"])) 
		inlineWiki 
		^(Function Name["EndOrderedListItem"])
	| O2 inlineWiki 
	-> ^(Function Name["BeginOrderedListItem"] ^(Assignment Key["level"] Value["2"])) 
		inlineWiki 
		^(Function Name["EndOrderedListItem"])
	| O3 inlineWiki 
	-> ^(Function Name["BeginOrderedListItem"] ^(Assignment Key["level"] Value["3"])) 
		inlineWiki 
		^(Function Name["EndOrderedListItem"])
	; 
	
 

inlineWiki
	: inlineWiki_i* NEWLINE!
	;

 
inlineWiki_i
	: function | wikiText | xmlElement | boldPart | italicPart 
	;

wikiText
	: Word -> Text[$Word.text]
	| WhiteSpaces -> Text[$WhiteSpaces.text]
	| (link) => link
	| InternetLink -> ^(Function Name["link"] ^(Assignment Key["href"] Value[$InternetLink.text]))
	| wikiSymbol 
	| StringLiteral -> Text[$StringLiteral.text]
	| PreText -> ^(Function Name["beginVerbatim"]) Text[$PreText.text] ^(Function Name["endVerbatim"])
	| WikiName -> ^(Function Name["wikilink"] ^(Assignment Key["wikiName"] Value[$WikiName.text]))
	| LineBreak -> ^(Function Name["linebreak"]) 
	;



boldPart
	: (~('!')) => '{b|' inlineWiki_i+ '}' -> ^(Function Name["beginBold"]) inlineWiki_i+ ^(Function Name["endBold"])
	;

italicPart
	: (~('!')) => '{i|' inlineWiki_i+ '}' -> ^(Function Name["beginItalic"]) inlineWiki_i+ ^(Function Name["endItalic"])
	;

wikiSymbol
	: (LBracket | RBracket	| Slash	| At | Colon | Eq | '"'	| Star | Comma | Dot | LParen | RParen) -> Text[$wikiSymbol.text]
	;

link: LBracket WhiteSpaces? link_i WhiteSpaces? linkLabel? RBracket 
		-> ^(Function Name["link"] 
			^(Assignment Key["href"] Value[$link_i.text])  
			linkLabel?)
	| LBracket WhiteSpaces? WikiName WhiteSpaces? linkLabel? RBracket
		-> ^(Function Name["wikilink"]
			 ^(Assignment Key["wikiName"] Value[$WikiName.text]) 
			 linkLabel?)
	;

linkLabel: ((Word | WikiName) WhiteSpaces?)+ -> ^(Assignment Key["label"] Value[$linkLabel.text])
	; 

link_i: Word | InternetLink
	;

qName
	: Word
	| (Word Colon Word) => Word Colon Word
	;

	
endTag:	CloseTagBegin qName Gt
	;	

xmlAttribute: Word Eq StringLiteral -> ^(XMLAttribute Key[$Word.text] Value[$StringLiteral.text])
	;

xmlElement
	: Lt qName (WhiteSpaces xmlAttribute)* Gt xmlContent endTag
	 -> ^(XMLElement Name[$qName.text] xmlAttribute* xmlContent)
	| Lt qName (WhiteSpaces xmlAttribute)* Slash Gt 
	 -> ^(XMLElement Name[$qName.text] xmlAttribute*)
	;

xmlContent
	: xmlContent_i*
	;

xmlContent_i
	: wikiText 
	| NEWLINE
	| xmlElement
	;

function
	: FunctionName RParen 
		-> ^(Function Name[$FunctionName.text])
	| FunctionName functionArgExprList RParen  
		-> ^(Function Name[$FunctionName.text] functionArgExprList)
	;


functionArgExprList
	: functionArgExpr (Comma! functionArgExpr)*
	;	

functionArgExpr
	: WhiteSpaces? functionArgExpr_i WhiteSpaces? -> functionArgExpr_i
	;

functionArgExpr_i
	: StringLiteral -> Argument[$StringLiteral.text]
	| WikiName -> ^(Function Name["wikilink"] ^(Assignment Key["wikiName"] Value[$WikiName.text]))
	| InternetLink -> Argument[$InternetLink.text]
	| arrayExpr
	| hashExpr
	| (Word) => Word -> Argument[$Word.text]
	| pathExpr
	| hashEntry_i
	| function
	;
	


pathExpr
	: Slash Word relativePathExpr -> Path[$pathExpr.text]
	| Word relativePathExpr -> Path[$pathExpr.text]
	;
	
relativePathExpr
	: (Slash Word)* 
	;	


	
arrayExpr
	: LBracket functionArgExprList? RBracket
	 -> ^(Array functionArgExprList?) 
	;	

hashExpr
	: LBrace hashEntryList? RBrace
	 -> hashEntryList?
	;	

hashEntryList
	: hashEntry (Comma! hashEntry)*
	;

hashEntry
	: WhiteSpaces? hashEntry_i WhiteSpaces? 
	 -> hashEntry_i 
	;		

hashEntry_i
	: Word WhiteSpaces? Eq WhiteSpaces? functionArgExpr_i
	  -> ^(Assignment Key[$Word.text] Value[$functionArgExpr_i.text])
	;		
	
	
	