lexer grammar Wiki;
options {
  language=Java;

}
@members {
  	private static HashMap<Integer, String> tokenTable = new HashMap<Integer, String>();
  	private static Logger _logger = Logger.getLogger(WikiLexer.class);
  	static
  	{
  		Properties p = new Properties();
  		try
        {
   		    URL wikiTokenFileURL = FileResource.find(WikiLexer.class, "Wiki.tokens");
   		    p.load(wikiTokenFileURL.openStream());

             for(Iterator it = p.keySet().iterator(); it.hasNext(); )
             {
				String tokenName = (String) it.next();
   	            int tokenType = Integer.parseInt(p.get(tokenName).toString());
                tokenTable.put(tokenType, tokenName);
             }
        }
        catch (IOException e)
        {
           _logger.error(e);
        }
  	}
  	public static String getTokenName(int tokenType)
   	{
   		return tokenTable.containsKey(tokenType) ? tokenTable.get(tokenType) : "UNKNOWN";
   	}
}
@header {
/*--------------------------------------------------------------------------
 *  Copyright 2007 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------------------
// xerial-wiki project
// 
// WikiLexer.java
// Since  2007/09/08 23:17:42
// 
//--------------------------------------------------

package org.xerial.wiki.impl;

import java.util.Iterator;
import java.util.HashMap;
import java.util.Properties;
import org.xerial.util.FileResource;
import org.xerial.util.log.Logger;
import java.io.IOException;
import java.net.URL;
}

T77 : '{b|' ;
T78 : '{i|' ;
T79 : '"' ;
T80 : '!' ;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 152
ML_COMMENT
	:   '/*' (options {greedy=false;} : .)* '*/' { channel=HIDDEN; }
 	;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 156
LINE_COMMENT
    : {getCharPositionInLine()==0}? => '//' ~('\n'|'\r')* NEWLINE { channel=HIDDEN; }
    ;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 160
NEWLINE
	: '\r'? '\n'
	;

 
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 165
fragment UpperLetter : 'A' .. 'Z';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 166
fragment LowerLetter: 'a' .. 'z';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 167
fragment Letter : UpperLetter | LowerLetter;
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 168
fragment Digit: '0' .. '9';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 169
fragment HexDigit: ('0' .. '9' | 'A' .. 'F' | 'a' .. 'f');
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 170
fragment UnicodeChar: ~('"'| '\\');
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 171
fragment StringChar :  UnicodeChar | EscapeSequence;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 173
fragment EscapeSequence
	: '\\' ('\"' | '\\' | '/' | 'b' | 'f' | 'n' | 'r' | 't' | 'u' HexDigit HexDigit HexDigit HexDigit)
	;


// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 178
fragment Int: '-'? ('0' | '1'..'9' Digit*);
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 179
fragment Frac: Dot Digit+;
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 180
fragment Exp: ('e' | 'E') ('+' | '-')? Digit+;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 182
fragment WhiteSpace: (' ' | '\t' | '\u000C');

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 184
LineBreak: '<BR>' (WhiteSpaces* NEWLINE)?;

//BlankLine: {getCharPositionInLine()==0}? => WhiteSpace* (NEWLINE | EOF);

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 188
WhiteSpaces: WhiteSpace+;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 190
H1: {getCharPositionInLine()==0}? => '=' { setText("H1"); };
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 191
H2: {getCharPositionInLine()==0}? => '==' { setText("H2"); };
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 192
H3: {getCharPositionInLine()==0}? => '===' { setText("H3"); };

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 194
I1: {getCharPositionInLine()==0}? => '*' { setText("I1"); };
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 195
I2: {getCharPositionInLine()==0}? => '**' { setText("I2"); };
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 196
I3: {getCharPositionInLine()==0}? => '***' { setText("I3"); };

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 198
O1: {getCharPositionInLine()==0}? => '+' { setText("I1"); };
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 199
O2: {getCharPositionInLine()==0}? => '++' { setText("I2"); };
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 200
O3: {getCharPositionInLine()==0}? => '+++' { setText("I3"); };




// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 205
fragment NameChar: Letter | Digit | Dot | '-' | '_' ;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 207
fragment Name: Letter NameChar*;

//StartTag: Lt n=Name Gt { setText(n.getText()); }; 
//EndTag: Lt Slash n=Name Gt { setText(n.getText()); };




// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 215
StringLiteral: '"' body=StringCharacters '"' { setText($body.text); };
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 216
fragment StringCharacters: StringChar*;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 218
fragment
ProtocolPrefix
	: ('http' | 'ftp' | 'https' | 'sftp' | 'rsync' ) '://'
	;
	
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 223
Dot: '.';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 224
LParen: '(';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 225
RParen: ')';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 226
LBracket: '[';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 227
RBracket: ']';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 228
LBrace: '{';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 229
RBrace: '}';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 230
Lt: '<';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 231
Gt: '>';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 232
Slash: '/';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 233
At: '@';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 234
Comma: ',';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 235
Colon: ':';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 236
Eq: '=';
// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 237
Star: '*';
//And: '&';
//Question: '?';
//Exclamation: '!';
//Sharp: '#';

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 243
fragment
Symbol
	: LBracket | RBracket | LBrace | RBrace | LParen | RParen 
	| Lt | Gt | At | Comma | Eq  | '"' | Star  | Slash
//	| And | Question | Exclamation | Sharp
	;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 250
fragment
WikiBlockSymbol
	: Star | Eq
	;


// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 256
fragment
URLChar: NameChar | At | Dot 
	;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 260
fragment
URLQueryChar: URLChar | Eq | '%'  
	;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 264
CloseTagBegin: Lt Slash; 



// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 268
fragment
WikiNameUnit: UpperLetter+ LowerLetter+ 
	;


// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 273
WikiName
	: WikiNameUnit WikiNameUnit
	| WikiNameUnit+ (Slash WikiNameUnit+)+ WikiNameUnit WikiNameUnit
	;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 278
InternetLink
	: (ProtocolPrefix) => ProtocolPrefix (~( WhiteSpace | '\n' | '\r' | ';' | LParen | RParen) | Digit)*
	| (~('!')) => Slash? URLChar+ (Colon Digit+)? (Slash URLChar*)* ('?' URLQueryChar*)?
	;	



// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 285
fragment FunctionID: Name (Colon Name)?;

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 287
FunctionName: (~('!')) => At FunctionID LParen { setText($FunctionID.text); } 	
	; 

	

// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 292
Word
	: {getCharPositionInLine() > 0 }? => (~(Symbol | WhiteSpace | '\n' | '\r'))+
	| {getCharPositionInLine() == 0}? => ~(WikiBlockSymbol | Symbol) (~(Symbol | WhiteSpace | '\n' | '\r'))*
	| ('!' WikiName) => '!' WikiName  { setText(getText().substring(1)); }
	| ('!' InternetLink) => '!' InternetLink  { setText(getText().substring(1)); }
	;
	



// $ANTLR src "F:\cygwin\home\leo\work\eclipse\workspace\xerial\xerial-wiki\src\main\java\org\xerial\wiki\impl\Wiki.g" 302
PreText
	: '<pre>' (options {greedy=false;} : .)* '</pre>'
	{ 
	  String tmp = getText(); 
	  setText(tmp.substring(5, tmp.length()-6));
	}
	| '{{' (options {greedy=false;} : .)* '}}'
	{
	  String tmp = getText(); 
	  setText(tmp.substring(2, tmp.length()-2));
	}
	;

