/*--------------------------------------------------------------------------
 *  Copyright 2008 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// BeginHeading.java
// Since: Apr 22, 2008 1:15:33 PM
//
// $URL: https://www.xerial.org/svn/project/XerialJ/trunk/xerial-wiki/src/main/java/org/xerial/wiki/plugin/BeginSectionTitle.java $
// $Author: leo $
//--------------------------------------
package org.xerial.wiki.plugin;

import java.util.List;

import org.xerial.wiki.TextRetriever;
import org.xerial.wiki.WikiBlock;
import org.xerial.wiki.WikiContext;
import org.xerial.wiki.WikiElement;
import org.xerial.wiki.WikiException;
import org.xerial.wiki.WikiPlugin;
import org.xerial.wiki.WikiWalker;
import org.xerial.wiki.WikiWriter;

/**
 * Begin Section Function
 * 
 * @author leo
 * 
 */
public class BeginSectionTitle implements WikiPlugin
{
    int level = -1;

    public int getLevel()
    {
        return level;
    }

    public void setLevel(int level)
    {
        this.level = level;
    }

    public void eval(String[] args, WikiContext context, WikiWriter writer) throws WikiException
    {
        writer.resetIndent(0);
        writer.beginSection(level);
        
        String anchorName = "";
        if(context.getParentWikiElement() != null)
        {
            TextRetriever sectionTitleRetriever = new TextRetriever();
            WikiWalker walker = new WikiWalker(sectionTitleRetriever);
            walker.addSkipElement(context.getCurrentWikiElement());
            walker.walk(context.getParentWikiElement());
            anchorName = sectionTitleRetriever.getText();
            anchorName = anchorName.replaceAll("[\\W\\p{Punct}]", "");
        }
        writer.beginAnchor(anchorName);
        writer.beginSectionTitle(level);
    }
    
    protected WikiElement getSibling(WikiElement parent, WikiElement currentElem)
    {
        switch(parent.getType())
        {
        case WikiBlock:
        {
            WikiBlock wikiBlock = WikiBlock.class.cast(parent);
            List<WikiElement> list = wikiBlock.getWikiElementList();
            for(int i=0; i<list.size(); i++)
            {
                WikiElement elem = list.get(i);
                if(elem.hashCode() == currentElem.hashCode())
                {
                    if(i < list.size() - 1)
                    {
                        return list.get(i+1);
                    }
                    else
                        return null;
                }
            }
            break;
        }
        }
        return null;
        
    }

}
