/*--------------------------------------------------------------------------
 *  Copyright 2007 Taro L. Saito
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// XerialJ
//
// LinkPlugin.java
// Since: Sep 16, 2007 12:23:01 PM
//
// $URL$
// $Author$
//--------------------------------------
package org.xerial.wiki.plugin;

import org.xerial.util.log.Logger;
import org.xerial.wiki.WikiContext;
import org.xerial.wiki.WikiErrorCode;
import org.xerial.wiki.WikiException;
import org.xerial.wiki.WikiPlugin;
import org.xerial.wiki.WikiWriter;

/**
 * Generate link
 * 
 * @author leo
 * 
 */
public class LinkPlugin implements WikiPlugin
{
    private static Logger _logger = Logger.getLogger(LinkPlugin.class);

    private String href = null;
    private String target = null;
    private String label = null;

    public LinkPlugin()
    {}

    public void eval(String[] args, WikiContext context, WikiWriter writer) throws WikiException
    {
        _logger.debug("link function");
        if (href == null)
            throw new WikiException(WikiErrorCode.SyntaxError, "no reference");
        if (label == null)
            label = href;

        assert (href != null);
        assert (label != null);

        if(href.endsWith(".jpg") || href.endsWith(".jpeg") || href.endsWith(".png") || href.endsWith(".gif") || href.endsWith(".tiff") || href.endsWith(".bmp"))
        {
            writer.beginImage(href, label);
            writer.endImage();
        }
        else
        {
            writer.beginLink(href, target);
            writer.text(label);
            writer.endLink();
        }
    }

    public String getHref()
    {
        return href;
    }

    public void setHref(String href)
    {
        this.href = href;
    }

    public String getTarget()
    {
        return target;
    }

    public void setTarget(String target)
    {
        this.target = target;
    }

    public String getLabel()
    {
        return label;
    }

    public void setLabel(String label)
    {
        this.label = label;
    }

}
